Package: rbiom
Type: Package
Title: Integrated Analysis and Visualization of Microbiome Data
Version: 3.1.0
Authors@R: c(
    person(
      "Daniel P.", "Smith",
      email   = "dansmith01@gmail.com",
      role    = c("aut", "cre"),
      comment = c(ORCID = "0000-0002-2479-2044")),
    person(
      "Alkek Center for Metagenomics and Microbiome Research", 
      role = c("cph", "fnd") ))
Description: 
    A toolkit for working with Biological Observation Matrix ('BIOM') files.
    Read/write all 'BIOM' formats. Compute rarefaction, alpha diversity, and 
    beta diversity (including 'UniFrac'). Summarize counts by taxonomic level. 
    Subset based on metadata. Generate visualizations and statistical analyses.
URL: https://cmmr.github.io/rbiom/, https://github.com/cmmr/rbiom
BugReports: https://github.com/cmmr/rbiom/issues
License: MIT + file LICENSE
Encoding: UTF-8
Depends: R (>= 4.3.0)
RoxygenNote: 7.3.3
Config/Needs/website: rmarkdown, phyloseq, npregfast, withr
Config/testthat/edition: 3
Imports: methods, mgcv, stats, utils, ape, dplyr, ecodive, emmeans,
        fillpattern, ggbeeswarm, ggnewscale, ggplot2, ggrepel, ggtext,
        jsonlite, magrittr, patchwork, pillar, plyr, readr, readxl,
        vegan
Suggests: cli, crayon, ggdensity, glue, h5lite, labeling, lifecycle,
        Matrix, openxlsx, optparse, pkgconfig, prettycode, R6, rlang,
        scales, testthat, tibble, tsne, uwot
NeedsCompilation: no
Packaged: 2026-05-07 16:43:38 UTC; Daniel
Author: Daniel P. Smith [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2479-2044>),
  Alkek Center for Metagenomics and Microbiome Research [cph, fnd]
Maintainer: Daniel P. Smith <dansmith01@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-08 08:00:02 UTC
Built: R 4.5.2; ; 2026-05-08 11:07:02 UTC; unix
