#' @export
ecodive::n_cpus

#' @export
plyr::`.`

#' @export
plyr::ddply

#' @export
plyr::ldply

#' @export
plyr::llply

#' @export
magrittr::`%>%`

#' @export
magrittr::`%<>%`

#' @export
pillar::glimpse

#' @export
stats::na.omit

#' @export
dplyr::left_join

#' @export
dplyr::mutate

#' @export
dplyr::pull

#' @export
dplyr::relocate

#' @export
dplyr::rename

#' @export
dplyr::select

#' @export
dplyr::slice

#' @export
dplyr::slice_head

#' @export
dplyr::slice_tail

#' @export
dplyr::slice_min

#' @export
dplyr::slice_max

#' @export
dplyr::slice_sample

#' @export
ecodive::list_metrics

#' @export
ecodive::match_metric

#' @export
ggplot2::ggsave
